@@ -14,12 +14,12 @@ typedef struct r_diff_op_t {
 	/* file A */
 	ut64 a_off;
 	const ut8 *a_buf;
-	int a_len;
+	ut32 a_len;
 
 	/* file B */
 	ut64 b_off;
 	const ut8 *b_buf;
-	int b_len;
+	ut32 b_len;
 } RDiffOp;
 
 //typedef struct r_diff_t RDiff;
@@ -30,7 +30,7 @@ typedef struct r_diff_t {
 	int delta;
 	void *user;
 	bool verbose;
-	bool levenstein;
+	int type;
 	int (*callback)(struct r_diff_t *diff, void *user, RDiffOp *op);
 } RDiff;
 
@@ -38,7 +38,7 @@ typedef int (*RDiffCallback)(RDiff *diff, void *user, RDiffOp *op);
 
 /* XXX: this api needs to be reviewed , constructor with offa+offb?? */
 #ifdef R_API
-R_API RDiff *r_diff_new();
+R_API RDiff *r_diff_new(void);
 R_API RDiff *r_diff_new_from(ut64 off_a, ut64 off_b);
 R_API RDiff *r_diff_free(RDiff *d);
 
@@ -47,8 +47,12 @@ R_API int r_diff_buffers_static(RDiff *d, const ut8 *a, int la, const ut8 *b, in
 R_API int r_diff_buffers_radiff(RDiff *d, const ut8 *a, int la, const ut8 *b, int lb);
 R_API int r_diff_buffers_delta(RDiff *diff, const ut8 *sa, int la, const ut8 *sb, int lb);
 R_API int r_diff_buffers(RDiff *d, const ut8 *a, ut32 la, const ut8 *b, ut32 lb);
+R_API char *r_diff_buffers_to_string(RDiff *d, const ut8 *a, int la, const ut8 *b, int lb);
 R_API int r_diff_set_callback(RDiff *d, RDiffCallback callback, void *user);
 R_API bool r_diff_buffers_distance(RDiff *d, const ut8 *a, ut32 la, const ut8 *b, ut32 lb, ut32 *distance, double *similarity);
+R_API bool r_diff_buffers_distance_myers(RDiff *diff, const ut8 *a, ut32 la, const ut8 *b, ut32 lb, ut32 *distance, double *similarity);
+R_API bool r_diff_buffers_distance_levenstein(RDiff *d, const ut8 *a, ut32 la, const ut8 *b, ut32 lb, ut32 *distance, double *similarity);
+R_API char *r_diff_buffers_unified(RDiff *d, const ut8 *a, int la, const ut8 *b, int lb);
 /* static method !??! */
 R_API int r_diff_lines(const char *file1, const char *sa, int la, const char *file2, const char *sb, int lb);
 R_API int r_diff_set_delta(RDiff *d, int delta);
